# Author: Kshiteesh Hegde, kshiteesh.hegde@gmail.com
# If there is a different author for a particular method, it is mentioned there
# Get a structured image of a network's subgraph by running:
# python get-signature/driver.py <network edgelist> <subgraph_size> <output_filename>

import networkx as nx
import sys
import matplotlib.pyplot as plt
from utils import sample_random_walk as rw
from utils import order_nodes as on


def main(data, ss, ofn):
    try:
        print('Reading the dataset')
        graph = nx.read_edgelist(data)
        print('Performing random walk of length: ' + ss)
        rwg = rw(graph, int(ss))
        print('Structuring the subgraph')
        adj = nx.attr_matrix(rwg, rc_order=on(rwg))
        if adj.shape[0] != int(ss):
            print('Did not find ' + ss + ' nodes, retrying...')
            main(data, ss, ofn)
        plt.axis('off')
        plt.imshow(adj, cmap='hot_r', interpolation='nearest')
        plt.savefig(ofn, bbox_inches='tight')
        print('The structured image of a sugbgraph of ' + data + ' has been saved at ' + ofn)
    except IOError:
        print('Unable to read graph. Please check the filename.')


if __name__ == '__main__':
    dataset = sys.argv[1]
    subgraph_size = sys.argv[2]
    out_filename = sys.argv[3]
    main(dataset, subgraph_size, out_filename)
