import networkx as nx
import random
import numpy as np


def get_next_node_by_ego_graph_size(graph, candidates):
    # Author: Ke Wu, wkcoke.work@gmail.com
    eccs = [nx.eccentricity(graph, v=node) for node in candidates]
    max_radius = max(eccs)
    for r in range(max_radius):
        max_cand = {}
        for node in candidates:
            key = len(nx.ego_graph(graph, node, radius=r))
            temp = max_cand.get(key, [])
            temp.append(node)
            max_cand[key] = temp
        max_val = max(max_cand.keys())
        candidates = max_cand[max_val]

        # max_val = max([len(nx.ego_graph(graph, node, radius=r)) for node in candidates])
        # candidates = [node for node in candidates if len(nx.ego_graph(graph, node, radius=r)) == max_val]

        if len(candidates) == 1:
            return candidates[0]  # skip out early if possible
    return random.choice(candidates)


def get_next_node_by_path_length(graph, ordering, candidates):
    # Author: Ke Wu, wkcoke.work@gmail.com
    # For each node, prune out nodes that are farther from the ordered nodes
    for i in range(len(ordering)):
        ordered_node = ordering[i]
        min_cand = {}
        for node in candidates:
            key = nx.shortest_path_length(graph, source=ordered_node, target=node)
            temp = min_cand.get(key, [])
            temp.append(node)
            min_cand[key] = temp
        min_dist = min(min_cand)
        candidates = min_cand[min_dist]

        # min_dist = min([nx.shortest_path_length(graph, source=ordered_node, target=node) for node in candidates])
        # candidates = [node for node in candidates if nx.shortest_path_length(graph, source=ordered_node,
        #                                                                      target=node) == min_dist]
        if len(candidates) == 1:
            return candidates[0]
    return get_next_node_by_ego_graph_size(graph, candidates)


def get_next_node(graph, ordering):
    # Author: Ke Wu, wkcoke.work@gmail.com
    candidates = [v for v in graph.nodes() if v not in ordering]
    return get_next_node_by_path_length(graph, ordering, candidates)


# Constructs a BFS-based ordering of nodes for the networkx graph
def order_nodes(graph):
    # Author: Ke Wu, wkcoke.work@gmail.com
    nodes = graph.nodes()
    ordering = []
    while len(ordering) != len(nodes):
        next_node = get_next_node(graph, ordering)
        ordering.append(next_node)
    return ordering


def sample_random_walk(graph, n):
    # Author: Ke Wu, wkcoke.work@gmail.com
    walk = set()
    firstnode = random.choice(graph.nodes())
    walk.add(firstnode)  # Start at a random node
    candidates = []
    for node in graph.neighbors(firstnode):
        if node not in walk:
            candidates.append(node)
    for i in range(1, n):
        if len(candidates) == 0:  # It's possible to start walking on a small connected component
            break
        nextnode = random.choice(candidates)
        walk.add(nextnode)
        # Remove the new node from the list of neighbors
        while nextnode in candidates:
            candidates.remove(nextnode)
        # Add the new neighbors from the new node:
        for node in graph.neighbors(nextnode):
            if node not in walk:
                candidates.append(node)
    edges = []
    for v1 in walk:
        for v2 in walk:
            if v2 in graph.neighbors(v1):
                edges.append((v1, v2))
    return nx.from_edgelist(edges)


def get_adj(graph):
    # graph = nx.read_edgelist(filename)
    return np.asarray(nx.attr_matrix(graph, rc_order=order_nodes(graph)))
